@@ -28,20 +28,40 @@
 
 void argv_factory_init(ArgvFactory *af)
 {
+    assert(af != NULL);
     af->size = 64;
     af->used = 0;
     af->argv = (char **)malloc(af->size * sizeof(char *));
+    assert(af->argv != NULL);
     *af->argv = NULL;
 }
 
 
 
+void argv_factory_release(ArgvFactory *af)
+{
+    assert(af != NULL);
+    assert(af->argv != NULL);
+    assert(af->used+1 <= af->size);
+    assert(af->argv[af->used] == NULL);
+    
+    free(af->argv);
+}
+
+
 void argv_factory_add(ArgvFactory *af, char *arg)
 {
+    assert(af != NULL);
+    assert(af->argv != NULL);
+    assert(af->used+1 <= af->size);
+    assert(af->argv[af->used] == NULL);
+    assert(arg != NULL);
+    
     if (af->used + 1 == af->size)
     {
         af->size *= 2;
         af->argv = realloc(af->argv, af->size * sizeof(char *));
+        assert(af->argv != NULL);
     }
     
     af->argv[af->used++] = strdup(arg);
