@@ -23,7 +23,7 @@
 #include "includes.h"
 #include "pgpopts.h"
 #include "argvfactory.h"
-
+#include "defaultkey.h"
 
 
 
@@ -89,7 +89,7 @@ static struct { char *keyword; int minlen; int	index; } mapping[] =
     { "comment",	4, PGP_LONG_OPT_COMMENT },
     { "completes_needed", 5, PGP_LONG_OPT_COMPLETESNEEDED },
     { "compress",	5, PGP_LONG_OPT_COMPRESS },
-    { "encrypttpself",	1, PGP_LONG_OPT_ENCRYPTTOSELF },
+    { "encrypttoself",	1, PGP_LONG_OPT_ENCRYPTTOSELF },
     { "force",		1, PGP_LONG_OPT_FORCE },
     { "interactive",	1, PGP_LONG_OPT_IGNORE },
     { "keepbinary",	1, PGP_LONG_OPT_NOSUPPORT },
@@ -612,6 +612,7 @@ char ** pgpopts_build_argv(Pgpopts *pgpopts)
         /* key commands */
         --opts.k;
         
+        build_batchmode(&af, &opts);
         build_force(&af, &opts);
         build_certdepth(&af, &opts);
         
@@ -688,51 +689,126 @@ char ** pgpopts_build_argv(Pgpopts *pgpopts)
         }
         else if (opts.v)
         {
-            if (opts.v == 1 && opts.c == 0)		/* -kv list keys (short version) */
-                argv_factory_add(&af, "--list-keys");
-            else if (opts.v >= 2 && opts.c == 0)	/* -kvv list keys (long version) */
-            {
+            if (opts.v >= 2)
                 argv_factory_add(&af, "--verbose");
-                argv_factory_add(&af, "--list-keys");
-                opts.v = 1;
-            }
-            else if (opts.v != 0 && opts.c == 1)	/* -kvc viev key fingerprint */
+            
+            if (opts.c == 0)
+                argv_factory_add(&af, "--list-keys");	/* -kv / -kvv */
+            else
             {
-                argv_factory_add(&af, "--verbose");
-                argv_factory_add(&af, "--fingerprint");
+                argv_factory_add(&af, "--fingerprint");	/* -kvc / -kvvc */
                 opts.c = 0;
             }
+            
+            if (opts.args_used == 1)
+                argv_factory_add(&af, opts.args[0]);
+            
+            if (opts.args_used >= 2)
+            {
+                fprintf(stderr, "BUG: Keyring argument not supported.\n");
+                return NULL;
+            }
         }
         else if (opts.c)			/* -kc check key signatures */
         {
             argv_factory_add(&af, "--check-sigs");
             opts.c = 0;
+            
+            if (opts.args_used == 1)
+                argv_factory_add(&af, opts.args[0]);
+            
+            if (opts.args_used >= 2)
+            {
+                fprintf(stderr, "BUG: Keyring argument not supported.\n");
+                return NULL;
+            }
         }
         else if (opts.r == 1 && opts.s == 0)	/* -kr delete key */
         {
             opts.r = 0;
             argv_factory_add(&af, "--delete-key");
+            
+            if (opts.args_used == 0)
+            {
+                fprintf(stderr, "Error: User ID required for deleting a key.\n");
+                return NULL;
+            }
+            
+            if (opts.args_used >= 2)
+            {
+                fprintf(stderr, "BUG: Keyring argument not supported.\n");
+                return NULL;
+            }
+            
+            argv_factory_add(&af, opts.args[0]);
         }
         else if (opts.r == 1 && opts.s == 1)	/* -krs remove signature from key */
         {
             opts.r = 0;
             opts.s = 0;
-            argv_factory_add(&af, "--edit-key");
+            
+            fprintf(stderr, "BUG: GnuPG doesn't support removing a key signature.\n");
+            return NULL;
         }
         else if (opts.r == 0 && opts.s == 1)	/* -ks add signature to key */
         {
             opts.s = 0;
             argv_factory_add(&af, "--edit-key");
+            
+            if (opts.args_used == 0)
+            {
+                fprintf(stderr, "Error: User ID required for signing a key.\n");
+                return NULL;
+            }
+            
+            if (opts.args_used >= 2)
+            {
+                fprintf(stderr, "BUG: Keyring argument not supported.\n");
+                return NULL;
+            }
+            
+            argv_factory_add(&af, opts.args[0]);
+            argv_factory_add(&af, "sign");
+            argv_factory_add(&af, "quit");
+            
         }
         else if (opts.e)			/* -ke edit key */
         {
             argv_factory_add(&af, "--edit-key");
             opts.e = 0;
+            
+            if (opts.args_used == 0)
+            {
+                fprintf(stderr, "Error: User ID required for editing a key.\n");
+                return NULL;
+            }
+            
+            if (opts.args_used >= 2)
+            {
+                fprintf(stderr, "BUG: Keyring argument not supported.\n");
+                return NULL;
+            }
+            
+            argv_factory_add(&af, opts.args[0]);
         }
         else if (opts.d)			/* -kd create key revocation certificate */
         {
             argv_factory_add(&af, "--gen-revoke");
             opts.d = 0;
+            
+            if (opts.args_used == 0)
+            {
+                fprintf(stderr, "Error: User ID required for generating key revokation certificate.\n");
+                return NULL;
+            }
+            
+            if (opts.args_used >= 2)
+            {
+                fprintf(stderr, "BUG: Keyring argument not supported.\n");
+                return NULL;
+            }
+            
+            argv_factory_add(&af, opts.args[0]);
         }
         else
         {
@@ -833,6 +909,23 @@ char ** pgpopts_build_argv(Pgpopts *pgpopts)
                 argv_factory_add(&af, opts.args[i]);
             }
             
+            if (opts.encrypttoself)
+            {
+                argv_factory_add(&af, "--remote-user");
+                if (opts.localuserid)
+                    argv_factory_add(&af, opts.localuserid);
+                else
+                {
+                    char *id = get_defaultkey();
+                    if (!id)
+                    {
+                        fprintf(stderr, "Error: Can't get default key user id.\n");
+                        return NULL;
+                    }
+                    argv_factory_add(&af, id);
+                }
+            }
+            
 #if 0
             if (!opts.f)
             {
